import os
import sys
import pathlib as pl
import subprocess
import pickledb


class Settings_Editor(object):
    """
    Class that manage all the parameters needed to
    generate the configuration file (setting.conf) for the engine.
    The configuration file path need to be specified.
    """

    def __init__(self, file_path: str):

        self.__op_system = sys.platform
        self.file_path = os.path.abspath(file_path)

        # self.db = pickledb.load("./carskit_api/settings_data.json", True)
        self.db = pickledb.load(
            os.path.join(
                pl.Path(self.file_path).parent,
                "settings_data.json"),
            True
        )

        self.__dataset_path = "None"

        # Talvez isso deixe de ser um path e vire só o nome da pasta
        # self.__results_foldername = os.path.join(
        #    pl.Path(self.file_path).parent.parent, "dataset/results/")
        self.__results_foldername = "results"

        self.__algorithm = "camf_cu"

        """
        Para as configurações de algoritmos específicos talvez seja necessário o uso de expressões regulares
        """

        self.__parameters = {
            "file_path": self.file_path,
            "dataset_path": self.__dataset_path,
            "results_foldername": self.__results_foldername,
            "algorithm": self.__algorithm,
            "topN": 10,
            "k_folds": 5,
            "random_seed": 1,
            "num_factors": 10,
            "num_max_iterations": 100,
            "learning_rate": 2e-2,
            "reg_lambda": 0.0001,
            "num_neighboors": 10,
            "similarity": "pcc",  # availables: pcc, cos, cos-binary, msd, cpc
        }

        self.__available_algorithms = [
            "itemknn",
            "userknn",
            "slopeone",
            "pmf",
            "bpmf",
            "biasedmf",
            "nmf",
            "svd++",
            "usersplitting",
            "itemsplitting",
            "uisplitting",
            "spf",
            "dcr",
            "dcw",
            "cptf",
            "camf_ci",
            "camf_cu",
            "camf_cuci",
            "cslim_ci",
            "cslim_cu",
            "cslim_cuci",
            "gcslim_cc",
            "cslim_ics",
            "cslim_lcs",
            "cslim_mcs",
            "gcslim_ics",
            "gcslim_lcs",
            "gcslim_mcs"
        ]

        if not self.db.get("parameters"):
            self.save_settings()
        self.load_settings()

    def set_dataset_path(self, path: str) -> str:
        """
        Define and validate the path to the dataset that is going
        to be used and returns a message about the operation's success.
        """

        if isinstance(path, str) and os.path.exists(path):
            self.__dataset_path = os.path.abspath(path)
            self.__parameters["dataset_path"] = self.__dataset_path

            return self.__dataset_path
        return "ERROR! Path Invalid."

    def get_dataset_path(self) -> str:
        return self.__dataset_path

    def set_results_foldername(self, name: str) -> str:
        """
        Define and validate the name for the results that the engine will generate
        and returns a message about the operation's success.
        """
        if isinstance(name, str):
            self.__results_foldername = name
            self.__parameters["results_foldername"] = self.__results_foldername

            return self.__results_foldername
        return "ERROR! Path Invalid."

    def get_results_foldername(self) -> str:
        return self.__results_foldername

    def set_algorithm(self, algo: str) -> str:
        """
        Define and validate the algorithm that is going
        to be used and returns a message about the operation's success.
        """

        if isinstance(algo, str) and algo != "":
            if algo.lower() in self.__available_algorithms:
                self.__algorithm = algo.lower()
                self.__parameters["algorithm"] = self.__algorithm

                return self.__algorithm
        return "ERROR! Algorithm not available."

    def get_algorithm(self) -> str:
        return self.__algorithm

    def set_parameter(self, key: str, value: str) -> str:
        """
        Define and validate a parameter for the next engine's execution
        and returns a message about the operation's success.
        """
        if key == "dataset_path":
            return self.set_dataset_path(value)
        elif key == "results_foldername":
            return self.set_results_foldername(value)
        elif key == "algorithm":
            return self.set_algorithm(value)
        else:
            if isinstance(key, str) and key in self.__parameters:
                self.__parameters[key] = value
                return "configuration setted"
            else:
                return "ERROR! operation invalid."

    def get_parameter(self, key: str) -> dict:
        if isinstance(key, str) and key in self.__parameters:
            return self.__parameters[key]
        else:
            return "ERROR! operation invalid."

    def load_settings(self) -> str:
        """
        Try to load the settings configuration from the file 'settings_data.db'
        and return an operation message.
        """
        if not self.db:
            return "ERROR! No Database connected!"

        self.__parameters = self.db.get("parameters")

        self.__file_path = self.__parameters["file_path"]
        self.__dataset_path = self.__parameters["dataset_path"]
        self.__results_foldername = self.__parameters["results_foldername"]
        self.__algorithm = self.__parameters["algorithm"]

        return "settings loaded"

    def save_settings(self) -> str:
        """
        Saves the current settings to the file 'settings_data.db'
        and returns a message about the operation's success.
        """

        status = self.db.set("parameters", self.__parameters)
        if status:
            self.db.dump()
        return "settings saved" if status else "saving error"

    def generate_file(self) -> str:
        """
        Generates the file 'settings.conf' with the current configuration
        and saves in the specified path.
        """

        if not os.path.exists(self.file_path):
            return "ERROR! Settings file's path do not exist!"
        if self.__dataset_path == "":
            return "ERROR! No dataset selected!"

        try:
            settings_file = open(self.file_path, "w")
        except FileNotFoundError:
            return "ERROR! The settings file path was not founded!"

        # VER NO GUIA DA ENGINE SE ELA CONSEGUE IDENTIFICAR O SISTEMA OPERACIONAL
        if settings_file.writable():
            settings_str = [
                f"dataset.ratings.wins={self.__parameters['dataset_path']}\n",
                f"dataset.ratings.lins={self.__parameters['dataset_path']}\n",
                "dataset.social.wins=-1\n",
                "dataset.social.lins=-1\n",
                "# options: -columns: (user, item, [rating, [timestamp]]) columns of rating data; -threshold: to binary ratings;\n",
                "# --time-unit [DAYS, HOURS, MICROSECONDS, MILLISECONDS, MINUTES, NANOSECONDS, SECONDS]\n",
                "ratings.setup=-threshold -1 -datatransformation 1 -fullstat -1\n",
                "# recommender=usersplitting -traditional biasedmf -minlenu 2 -minleni 2\n",
                f"recommender={self.__algorithm}\n",
                "# main option: 1. test-set -f test-file-path; 2. cv (cross validation) -k k-folds [-p on, off]\n",
                "# 3. leave-one-out; 4. given-ratio -r ratio;\n",
                "# other options:  [--rand-seed n] [--test-view all] [--early-stop loss, MAE, RMSE]\n",
                "# evaluation.setup=cv -k 5 -p on --rand-seed 1 --test-view all --early-stop RMSE\n",
                "# evaluation.setup=given-ratio -r 0.8 -target r --test-view all --rand-seed 1\n",
                "# main option: is ranking prediction\n",
                "# other options: -ignore NumOfPopularItems\n",
                f"evaluation.setup=cv -k {self.__parameters['k_folds']} -p on --rand-seed {self.__parameters['random_seed']} --test-view all\n",
                f"item.ranking=on -topN {self.__parameters['topN']}\n",
                f"output.setup=-folder {self.__parameters['results_foldername']} -verbose on, off --to-file results_all.txt\n",
                "# Guava cache configuration\n",
                "guava.cache.spec=maximumSize=200,expireAfterAccess=2m\n",
                "########################### Model-based Methods ############################\n"
                f"num.factors={self.__parameters['num_factors']}\n",
                f"num.max.iter={self.__parameters['num_max_iterations']}\n",
                "# options: -bold-driver, -decay ratio, -moment value\n",
                f"learn.rate={self.__parameters['learning_rate']} -max -1 -bold-driver\n",
                f"reg.lambda={self.__parameters['reg_lambda']} -c 0.001\n",
                "#reg.lambda=10 -u 0.001 -i 0.001 -b 0.001 -s 0.001 -c 0.001\n",
                "# probabilistic graphic models\n",
                "pgm.setup=-alpha 2 -beta 0.5 -burn-in 300 -sample-lag 10 -interval 100\n",
                "########################### Memory-based Methods ##########################\n"
                "# similarity method: PCC, COS, COS-Binary, MSD, CPC, exJaccard; -1 to disable shrinking;\n",
                f"similarity={self.__parameters['similarity']}\n",
                "num.shrinkage=-1\n",
                "# neighborhood size; -1 to use as many as possible.\n",
                f"num.neighbors={self.__parameters['num_neighboors']}\n",
                "########################## Method-specific Settings ########################\n"
                "AoBPR=-lambda 0.3\n",
                "BUCM=-gamma 0.5\n",
                "BHfree=-k 10 -l 10 -gamma 0.2 -sigma 0.01\n",
                "FISM=-rho 100 -alpha 0.4\n",
                "Hybrid=-lambda 0.5\n",
                "LDCC=-ku 20 -kv 19 -au 1 -av 1 -beta 1\n",
                "PD=-sigma 2.5\n",
                "PRankD=-alpha 20\n",
                "RankALS=-sw on\n",
                "RSTE=-alpha 0.4\n",
                "DCR=-wt 0.9 -wd 0.4 -p 5 -lp 2.05 -lg 2.05\n",
                "DCW=-wt 0.9 -wd 0.4 -p 5 -lp 2.05 -lg 2.05 -th 0.8\n",
                "SPF=-i 0 -b 5 -th 0.9 -f 10 -t 100 -l 0.02 -r 0.001\n",
                "SLIM=-l1 1 -l2 1 -k 1\n",
                "CAMF_LCS=-f 10\n",
                "CSLIM_C=-lw1 1 -lw2 5 -lc1 1 -lc2 5 -k 3 -als 0\n",
                "CSLIM_CI=-lw1 1 -lw2 5 -lc1 1 -lc2 1 -k 1 -als 0\n",
                "CSLIM_CU=-lw1 1 -lw2 0 -lc1 1 -lc2 5 -k 10 -als 0\n",
                "CSLIM_CUCI=-lw1 1 -lw2 5 -lc1 1 -lc2 5 10 -1 -als 0\n",
                "GCSLIM_CC=-lw1 1 -lw2 5 -lc1 1 -lc2 5 -k -1 -als 0\n",
                "CSLIM_ICS=-lw1 1 -lw2 5 -k 1 -als 0\n",
                "CSLIM_LCS=-lw1 1 -lw2 5 -k 1 -als 0\n",
                "CSLIM_MCS=-lw1 -20000 -lw2 100 -k 3 -als 0\n",
                "GCSLIM_ICS=-lw1 1 -lw2 5 -k 10 -als 0\n",
                "GCSLIM_LCS=-lw1 1 -lw2 5 -k -1 -als 0\n",
                "GCSLIM_MCS=-lw1 1 -lw2 5 -k -1 -als 0\n",
                "FM=-lw 0.01 -lf 0.02\n"
            ]
            for i in range(len(settings_str)):
                settings_file.write(settings_str[i])

            settings_file.close()
            # print("#########")
            # print(f"File name: { os.path.basename(self.file_path) }")
            # print("#########")
            return "The settings file was generated!"
        else:
            return "ERROR! The file is not readable!"

    def __del__(self):
        """
        Saves the settings when the class is dealocated.
        """
        self.save_settings()



from  run_carskit import Runner

def test_run_engine():
    runner = Runner('C:\\Users\\Waguinho\\Documents\\pesquisa\\CARSKit_Interface\\source\\CARSKit')
    OK_CODE = 0
    assert runner.run_engine() == OK_CODE


import os
import subprocess


class Runner(object):
    """
    Class that runs the CARSKit java file.
    It is only need the complete or relative path to the file to run it.
    """

    def __init__(self, javaFile_path: str):
        self.javaFile_path = javaFile_path
        self.current_path = os.path.abspath(os.getcwd())

    def run_engine(self) -> str:
        """
        Runs the Java file and return the output of the execution.
        """
        os.chdir(self.javaFile_path)
        output = subprocess.call(
            ['java', '-jar', 'CARSKit-v0.3.5.jar', '-c', 'setting.conf'])
        os.chdir(self.current_path)
        return output



import logging
import os
import pathlib as pl
from cliff.command import Command
from ..editors import Settings_Editor


class Settings(Command):
    """Edit the Engine's Settings"""

    def get_parser(self, prog_name):
        """Settings argument parsing."""
        parser = super(Settings, self).get_parser(prog_name)
        group = parser.add_mutually_exclusive_group()

        group.add_argument(
            "-s", "--set",
            help="set a new value for a specific configuration",
            action="store_true",
        )
        group.add_argument(
            "-g", "--get",
            help="get the value of the configuration specified",
            action="store_true",
        )

        parser.add_argument(
            "field",
            help="The name of the configuration field",
            action="store"
        )

        parser.add_argument(
            "--value",
            help="The new value for the configuration field specified. When the --get flag is setted this argument is disabled.",
            action="store"
        )

        return parser

    def take_action(self, parsed_args):
        """Settings action"""
        output = "test"

        conf_file_path = pl.Path(
            os.path.dirname(os.path.abspath(__file__)))

        conf_file_path = os.path.join(
            conf_file_path.parent, "carskit/test.conf")

        settings_editor = Settings_Editor(conf_file_path)

        if parsed_args.get:
            param = settings_editor.get_parameter(parsed_args.field)
            return str(param)

        if parsed_args.set:
            if not parsed_args.value:
                return "No value setted. Operation canceled."

            value = parsed_args.value
            result = settings_editor.set_parameter(parsed_args.field, value)
            return result

        return output



import pytest
from data_manager import Model_Statistics_Manager, Recommendations_Manager

@pytest.fixture
def sample_Model_Statistics_Manager():
    return Model_Statistics_Manager()

@pytest.fixture
def sample_Recommendations_Manager():
    return Recommendations_Manager()

@pytest.fixture
def sample_stats_file():
    stats_file = open("./source/datasets/results/sample@statistics.txt")
    return stats_file

@pytest.fixture
def sample_recommendations_file():
    rec_file = open("./source/datasets/results/sample_recommendations.txt")
    return rec_file

"""
##############################
Model_Statistics_Manager Tests
##############################
"""

def non_formatted_valid_stats_data():
    return {
        'average_ratings': '3,328892',
        'context_conditions': '10(time:3,location:3,companion:4)',
        'context_dimensions': '3(time,location,companion)',
        'context_situations': '13',
        'data_density': '1,8230%',
        'item_mount': '79',
        'median_ratings': '4,000000',
        'mode_ratings': '5,000000',
        'rate_amount': '5035',
        'scale_distributions': '[1.0x829,2.0x625,4.0x1209,5.0x1367,3.0x1005]',
        'standard_deviation_ratings': '1,414777',
        'user_amount': '97'
    }

def non_formated_invalid_stats_data():
    return {
        'average_ratings': '3,328892',
        'context_conditions': '10(time:3,location:3,companion:4)',
        'context_dimensions': '3(time,location,companion)',
        'context_situations': '13',
        'data_density': '1,8230%',
        'item_mount': '79',
        'mode_ratings': '5,000000',
        'rate_amount': '5035',
        'scale_distributions': '[1.0x829,2.0x625,4.0x1209,5.0x1367,3.0x1005]',
        'user_amount': '97'
    }

########## format_stats(self, stats_data)


@pytest.mark.parametrize("stats_data, expected_result", [
    (
        non_formatted_valid_stats_data(),
        "statistics data successfully formatted!"
    ),
    (
        non_formated_invalid_stats_data(),
        "ERROR! statistics data dict not supported"
    )
])
def test_format_stats(sample_Model_Statistics_Manager, stats_data, expected_result):
    assert sample_Model_Statistics_Manager.format_stats(stats_data) == expected_result


########## generate_statistic_data(self, statistic_file):


def test_generate_statistic_data(sample_Model_Statistics_Manager, sample_stats_file):
    assert sample_Model_Statistics_Manager.generate_statistic_data(sample_stats_file) == non_formatted_valid_stats_data()


########## save_statistics(self):
@pytest.mark.parametrize("file_path, expected_result",[
    ("./source/datasets/results/sample@statistics.txt",
    "The file execution_statistics.json was successfully created!"),
    ("./source/datasets/results/avbnbqd.txt",
    "ERROR! The file ./source/datasets/results/avbnbqd.txt could not be opened.")
])
def test_save_statistics(sample_Model_Statistics_Manager, file_path, expected_result):
    assert sample_Model_Statistics_Manager.save_statistics(file_path) == expected_result


"""
##############################
Recommendations_Manager Tests
##############################
"""


########## save_recommendations(self):

@pytest.mark.parametrize("results_file, expected_result", [
    (
        "./source/datasets/results/sample_recommendations.txt",
    "The file sample_recommendations.json was successfully created!"
    ),
    (
        "./source/datasets/results/edcveadw.txt",
        "ERROR! The file ./source/datasets/results/edcveadw.txt could not be opened."
    )
])
@pytest.mark.data_writing
def test_save_recommendations(sample_Recommendations_Manager, results_file, expected_result):
    assert sample_Recommendations_Manager.save_recommendations(results_file) == expected_result


import sys
import os
import pathlib as pl
from cliff.app import App
from cliff.commandmanager import CommandManager
from ..editors import Settings_Editor


class CAPI_App(App):
    def __init__(self):
        super(CAPI_App, self).__init__(
            description="An API for the CARSKit engine",
            version="0.0.1",
            command_manager=CommandManager("capi"),
            deferred_help=True,
        )

    def initialize_app(self, argv):
        self.LOG.debug("initialize_app")
        app_path = pl.Path(os.path.dirname(os.path.abspath(__file__))).parent
        settings_editor = Settings_Editor(
            os.path.join(app_path, "carskit/setting.conf")
        )
        settings_editor.save_settings()


def main(argv=sys.argv[1:]):
    app = CAPI_App()
    return app.run(argv)


if __name__ == "__main__":
    sys.exit(main(sys.argv[1:]))



from cliff.command import Command
import logging


class Message(Command):
    """A basic command"""

    def get_parser(self, prog_name):
        """Message argument parsing."""
        parser = super(Message, self).get_parser(prog_name)
        group = parser.add_mutually_exclusive_group()

        group.add_argument(
            "--lowercase",
            help="print result in lower case",
            action="store_true",
        )

        group.add_argument(
            "--uppercase",
            help="print result in upper case",
            action="store_true",
        )

        return parser

    def take_action(self, parsed_args):
        """Command action"""
        result = "Hello World!"

        if parsed_args.lowercase:
            return result.lower()

        if parsed_args.uppercase:
            return result.upper()

        return result



import pickledb

class Database(object):
    def __init__(self, db_path="": str):
        self.database_path = db_path
        self.database_obj = None
    
    # Futuramente pode ser trocada por self.connect_database()
    def load_database(self) -> bool:
        self.database_obj = pickledb.load(self.database_path, True)
        if self.database_obj:
            return True
        return False

    def create(self, query: str) -> bool:
        return self.database_obj.set(query, None)

    def read(self, query: str) -> bool:
        return self.database_obj.get(query)

    def update(self, query: str, value: any) -> bool:
        return self.database_obj.set(query, value)

    def delete(self, query: str) -> bool:
        return self.database_obj.rem(query)

    def save(self) -> bool:
        return self.database_obj.dump()

    def exist(self, query: str) -> bool:
        return self.database_obj.exists(query)

    # def disconnect_database(self):
    #    pass




import os
import pytest
from edit_settings import Settings_Editor

@pytest.fixture
def sample_settings_editor():
    return Settings_Editor()

@pytest.fixture
def available_algorithms():
    return [
        "itemknn",
        "userknn",
        "slopeone",
        "pmf",
        "bpmf",
        "biasedmf",
        "nmf",
        "svd++",
        "usersplitting",
        "itemsplitting",
        "uisplitting",
        "spf",
        "dcr",
        "dcw",
        "cptf",
        "camf_ci",
        "camf_cu",
        "camf_cuci",
        "cslim_ci",
        "cslim_cu",
        "cslim_cuci",
        "gcslim_cc",
        "cslim_ics",
        "cslim_lcs",
        "cslim_mcs",
        "gcslim_ics",
        "gcslim_lcs",
        "gcslim_mcs"
    ]

""" Setters """

@pytest.mark.parametrize("dataset_path, expected_result", [
    (12, False),
    ("", False),
    ("dawdwfw", False),
    ("./source/datasets/ratings.txt", True),
    (os.path.abspath("./source/datasets/ratings.txt"), True)
])
def test_set_dataset_path(sample_settings_editor, dataset_path, expected_result):
    assert sample_settings_editor.set_dataset_path(dataset_path) == expected_result

@pytest.mark.parametrize("results_path, expected_result", [
    (12, False),
    ("", False),
    ("dawdwfw", False),
    ("./source/datasets/results/", True),
    (os.path.abspath("./source/datasets/results/"), True)
])
def test_set_results_path(sample_settings_editor, results_path, expected_result):
    assert sample_settings_editor.set_results_path(results_path) == expected_result


@pytest.mark.parametrize("algorithm, expected_result", [
    (41231, False),
    ("", False),
    ("goieoawpd", False),
    ("fdp", False),
    ("CAMF_CU", True),
    ("camf_cu", True)
])
def test_set_algorithm(sample_settings_editor, algorithm, expected_result):
    assert sample_settings_editor.set_algorithm(algorithm) == expected_result


@pytest.mark.parametrize("parameter_key, parameter_value,expected_result", [
    ("topN", 20, True),
    ("k_folds", 3, True),
    ("random_seed", 2, True),
    ("num_factors", 12, True),
    ("num_max_iterations", 200, True),
    ("learning_rate", 2e-3, True),
    ("reg_lambda", 0.0004, True),
    ("num_neighboors", 5, True),
    ("similarity", "cos", True)
])
def test_set_parameter(sample_settings_editor, parameter_key, parameter_value, expected_result):
    assert sample_settings_editor.set_parameter(parameter_key, parameter_value) == expected_result
""" Getters """

def test_get_dataset_path(sample_settings_editor):
    sample_settings_editor.set_dataset_path(os.path.abspath("./source/datasets/ratings.txt"))
    assert sample_settings_editor.get_dataset_path() == os.path.abspath("./source/datasets/ratings.txt")

def test_get_results_path(sample_settings_editor):
    sample_settings_editor.set_results_path(os.path.abspath("./source/datasets/results/"))
    assert sample_settings_editor.get_results_path() == os.path.abspath("./source/datasets/results/")

def test_get_algorithm(sample_settings_editor, available_algorithms):
    assert sample_settings_editor.get_algorithm() in available_algorithms


@pytest.mark.parametrize("parameter_key, expected_result", [
    ("topN", 10),
    ("k_folds", 5),
    ("random_seed", 1),
    ("num_factors", 10),
    ("num_max_iterations", 100),
    ("learning_rate", 2e-2),
    ("reg_lambda", 0.0001),
    ("num_neighboors", 10),
    ("similarity", "pcc")
])
def test_get_parameters(sample_settings_editor, parameter_key, expected_result):
    assert sample_settings_editor.get_parameter(parameter_key) == expected_result

################

def test_load_settings(sample_settings_editor):
    ok_messages = [
        "no previous settings, DB initialized",
        "settings loaded"
    ]
    assert sample_settings_editor.load_settings() in ok_messages

def test_save_settings(sample_settings_editor):
    assert sample_settings_editor.save_settings() == True

@pytest.mark.parametrize("file_path, expected_result", [
    ("./source/datasets/ratings.txt", "The settings file was generated!"),
    ("./source/datasets/avnoaf.txt", "ERROR! The settings file path was not founded!")
])
def test_generate_file(sample_settings_editor, file_path, expected_result):
    sample_settings_editor.set_dataset_path(os.path.abspath(file_path))
    assert sample_settings_editor.generate_file() == expected_result


import logging
import os
import glob
import datetime
import pathlib as pl
from cliff.command import Command

from ..runner import Runner
from ..editors import Settings_Editor
from ..managers import Model_Statistics_Manager, Recommendations_Manager


class RunEngine(Command):
    """Run the CARSKit engine"""

    # CRIAR UMA FLAG PARA MEDIR O TEMPO DE EXECUCAO DA ENGINE
    # CRIAR UMA FLAG PARA ESPECIFICAR O TIPO DE SAIDA DA ENGINE

    def get_parser(self, prog_name):
        """RunEngine argument parsing."""
        parser = super(RunEngine, self).get_parser(prog_name)
        # group = parser.add_mutually_exclusive_group()

        #     group.add_argument(
        #         "--lowercase",
        #         help="print result in lower case",
        #         action="store_true",
        #     )

        return parser

    def latest_execution_data(self, app_path, dataset_path, results_foldername):
        data_filenames = glob.glob(
            os.path.join(
                app_path,
                os.path.dirname(dataset_path),
                f"{results_foldername}/*",
            )
        )

        data_filenames.sort(key=os.path.getctime, reverse=True)
        # return list(map(os.path.basename, data_filenames))
        return data_filenames

    def extract_stat_data(self, stats_mngr, stats_filepath):
        stats_file = open(stats_filepath)
        execution_stats = stats_mngr.generate_statistic_data(
            stats_file
        )
        stats_file.close()

        return execution_stats

    def extract_recommendations(self, recs_mngr, recs_filepath):
        recommendations_file = open(recs_filepath)
        recommendations = recs_mngr.generate_recommendations_data(
            recommendations_file
        )
        recommendations_file.close()

        return recommendations

    def take_action(self, parsed_args):
        """RunEngine action"""
        #output = "Is Running!"

        app_path = pl.Path(os.path.dirname(os.path.abspath(__file__))).parent
        engine_folder_path = os.path.join(app_path, "carskit/")
        conf_file_path = os.path.join(app_path, "carskit/setting.conf")

        settings_editor = Settings_Editor(conf_file_path)
        runner = Runner(engine_folder_path)

        current_time = datetime.datetime.today().strftime('%Y-%m-%d-%H:%M:%S')

        stats_filename = f"execution@{current_time}.json"
        recs_filename = f"recomendations@{current_time}.json"

        stats_mgn = Model_Statistics_Manager(
            settings_editor.get_results_foldername(),
            stats_filename,
        )

        recs_mgn = Recommendations_Manager(
            settings_editor.get_results_foldername(),
            recs_filename
        )

        status = settings_editor.generate_file()

        if status == "The settings file was generated!":

            runner.run_engine()

            execution_datafiles = self.latest_execution_data(
                app_path,
                settings_editor.get_dataset_path(),
                settings_editor.get_results_foldername()
            )

            target_path = pl.Path(execution_datafiles[0]).parent.parent

            execution_stats = self.extract_stat_data(
                stats_mgn,
                execution_datafiles[0]
            )

            recommendations = self.extract_recommendations(
                recs_mgn,
                execution_datafiles[1]
            )

            stats_mgn.save_statistics(
                execution_stats,
                os.path.join(target_path, stats_filename)
            )
            recs_mgn.save_recommendations(
                recommendations,
                os.path.join(target_path, recs_filename)
            )

            return
        else:
            output = "An error has occured, execution canceled."

        return



from setuptools import setup, find_packages

setup(
    # Application name
    name="carskit_api",
    # Version number
    version="0.0.4",
    description="An API for the CARSKit engine",
    # Application author details
    author="WagnoLeaoSergio",
    author_email="wagnoleao@gmail.com",
    # Dependencies
    install_requires=["cliff", "pickleDB"],
    # Packages
    packages=find_packages(),
    python_requires='>=3.6',

    # Details
    #url  = ""
    include_package_data=True,

    # License
    license="LICENSE.txt",

    # ReadMe
    long_description=open("README.md").read(),

    # Package console entry points
    entry_points={
        "console_scripts": ["capi = carskit_api.cli.main:main"],
        "capi": [
            "run = carskit_api.cli.run:RunEngine",
            "settings = carskit_api.cli.settings:Settings",
        ],
    },
)



import re
import json
import os
import pathlib as pl
import datetime

"""
    Dependendo do quão parecido essas classes ficarem
        uma super classe Data_Manager talvez seja criada...

    class DataSet_Manager(object):
        def __init__(self):
            coisas....
"""


class Model_Statistics_Manager(object):
    """
    Class responsible for managing the statistics created after
    an execution of the engine. Requires the name of the statistics file.
    """

    def __init__(self, output_folder: str, statistics_filename="sample@statistics"):
        self.statistics_filename = statistics_filename
        self.output_folder = output_folder
        self.current_path = pl.Path(
            os.path.dirname(os.path.abspath(__file__)))

    def generate_statistic_data(self, statistic_file) -> dict:
        """
        Extracts the essencial data from the statistics file specified and
        returns it as an dictionary.
        """

        fields_founded = False
        stats_data = {
            " * User amount": {
                "field_name": "user_amount",
                "value": 0
            },
            " * Item amount": {
                "field_name": "item_mount",
                "value": 0
            },
            " * Rate amount": {
                "field_name": "rate_amount",
                "value": 0
            },
            " * Context dimensions": {
                "field_name": "context_dimensions",
                "value": 0
            },
            " * Context conditions": {
                "field_name": "context_conditions",
                "value": 0
            },
            " * Context situations": {
                "field_name": "context_situations",
                "value": 0
            },
            " * Data density": {
                "field_name": "data_density",
                "value": 0
            },
            " * Scale distribution": {
                "field_name": "scale_distributions",
                "value": ""
            },
            " * Average value of all ratings": {
                "field_name": "average_ratings",
                "value": 0
            },
            " * Standard deviation of all ratings": {
                "field_name": "standard_deviation_ratings",
                "value": 0
            },
            " * Mode of all rating values": {
                "field_name": "mode_ratings",
                "value": 0
            },
            " * Median of all rating values": {
                "field_name": "median_ratings",
                "value": 0
            }
        }

        try:
            for line in statistic_file:
                field = re.findall(" [*]? .*", line)
                if field != []:
                    field = field[0]
                    field = re.split(":", field, 1)

                    if field[0] in stats_data:
                        stats_data[field[0]]["value"] = re.sub(
                            " ", "", field[1])
                        fields_founded = True
        except KeyError as e:
            return e

        if not fields_founded:
            return "ERROR! stats file not suporrted"

        labeled_data = {}
        for field in stats_data.items():
            labeled_data[field[1]['field_name']] = field[1]['value']

        return labeled_data

    def format_stats(self, stats_data: dict) -> str:
        """
        Format the dictionary of statistical data specified
        and returns a message about the operation's success.
        """

        if "average_ratings" in stats_data and "median_ratings" in stats_data \
                and "mode_ratings" in stats_data and "standard_deviation_ratings" in stats_data \
                and "scale_distributions" in stats_data:

            stats_data["average_ratings"] = re.sub(
                ",", ".", stats_data["average_ratings"])
            stats_data["median_ratings"] = re.sub(
                ",", ".", stats_data["median_ratings"])
            stats_data["mode_ratings"] = re.sub(
                ",", ".", stats_data["mode_ratings"])
            stats_data["standard_deviation_ratings"] = re.sub(
                ",", ".", stats_data["standard_deviation_ratings"])

            stats_data["scale_distributions"] = re.split(
                ",",  stats_data["scale_distributions"])

            stats_data["scale_distributions"][0] = re.sub(
                r"\[|\]", "", stats_data["scale_distributions"][0])
            stats_data["scale_distributions"][len(stats_data["scale_distributions"]) - 1] = re.sub(
                r"\[|\]", "", stats_data["scale_distributions"][len(
                    stats_data["scale_distributions"]) - 1]
            )

            return "statistics data successfully formatted!"

        return "ERROR! statistics data dict not supported"

    def save_statistics(self, stats_data, stats_file_path: str = None) -> str:
        """
        Generate and save the data extracted from the statistic file specified
        and returns a message about the operation's success.
        """

        results_folder = pl.Path(os.path.dirname(stats_file_path)).parent

        if not os.path.exists(results_folder):
            os.mkdir(results_folder)

        file_name = os.path.basename(stats_file_path)

        try:
            statistics_file = open(stats_file_path, "w+")
        except FileNotFoundError:
            return f"ERROR! The file {stats_file_path} could not be opened."

        if statistics_file.writable():

            self.format_stats(stats_data)
            json.dump(stats_data, statistics_file)
            statistics_file.close()

            return f"The file {file_name + '.json'} was successfully created!"

        return f"ERROR! The file {stats_file_path} is not readable."


class Recommendations_Manager(object):
    """
    Class responsible for managing the recommendations generated after
    an execution of the engine. Requires the name of the recommendation file.
    """

    def __init__(self, output_folder: str, result_filename="sample_recommendations"):

        self.current_path = pl.Path(
            os.path.dirname(os.path.abspath(__file__)))
        self.output_folder = output_folder
        self.result_filename = result_filename

    def get_line_header(self, line: str):
        """
        Returns the user's id and 
        a list of his contexts from the specified line.
        """

        contexts = []

        s_line = line.split(sep=',')
        user = s_line[0]
        context_str = s_line[1]
        context_list = re.findall(".*?;|.*?: ", context_str)

        for context in context_list:
            s_context = context.split(sep=":")
            context_name = s_context[0]
            context_value = s_context[1]

            context_name = re.sub(" ", "", context_name)
            context_value = re.sub(" ", "", context_value)
            context_value = re.sub(";", "", context_value)

            contexts.append((context_name, context_value))

        return user, contexts

    def get_line_recommendations(self, line: str) -> list:
        """
        Returns a list of items and ratings from the
        specified line.
        """

        rec_strings = re.findall(r"\(.*?\)", line)
        rec_data = []
        for rec in rec_strings:

            s_rec = re.sub("[(]|[)]", "", rec)
            s_rec = re.sub(" ", "", s_rec)
            s_rec = s_rec.split(sep=',')
            item = s_rec[0]
            rating = s_rec[1]

            rec_data.append((item, rating))

        return rec_data

    def create_context_combinations(self, contexts: list) -> str:
        """
        Returns the context combination string from a 
        specified list of contexts.
        """

        ctx = ""
        for i in range(len(contexts) - 1):
            ctx = ctx + contexts[i][1] + ", "
        ctx = ctx + contexts[-1][1]

        return ctx

    def generate_recommendations_data(self, recs_files) -> dict:
        """
        Returns a dictionary of items recommendations for each user
        for each combination  for context(s) from a specified file.
        """

        results_data = {
            "users": {}
        }

        header = recs_files.readline()
        for line in recs_files:

            user, contexts = self.get_line_header(line)
            recommendations = self.get_line_recommendations(line)
            context_combination = self.create_context_combinations(contexts)

            if user not in results_data["users"]:
                results_data["users"][user] = {}
            if context_combination not in results_data["users"][user]:
                results_data["users"][user][context_combination] = []

            results_data["users"][user][context_combination] = recommendations

        return results_data

    def save_recommendations(self, recs_data, recs_file_path: str = None) -> str:
        """
        Extract the recommendations from a file which his path need to be specified,
        saves them as a JSON file and returns a message about the operations success.
        """

        results_folder = pl.Path(os.path.dirname(recs_file_path)).parent

        if not os.path.exists(results_folder):
            os.mkdir(results_folder)

        file_name = os.path.basename(recs_file_path)

        try:
            recommendations_file = open(recs_file_path, "w+")
        except FileNotFoundError:
            return f"ERROR! The file {recs_file_path} could not be opened."

        if recommendations_file.writable():

            json.dump(recs_data, recommendations_file)
            recommendations_file.close()

            return f"The file {file_name + '.json'} was successfully created!"

        return f"ERROR! The file {recs_file_path} is not readable."
